/***************************************************************************
                          ezaddress.cpp  -  description
                             -------------------
    begin                : Tue Aug 17 1999
    copyright            : (C) 1999 by Joe Nelson
    email                : 3nelsons@pressenter.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "easyaddress.h"

EasyAddress::EasyAddress(const string& serverNameOrIP) throw(EasyNetError) {
	struct in_addr host;
	bool isName; // if we're going to throw an exception, we want to know whether we
	             // used gethostbyname() or gethostbyaddr(). This flag holds that info.

	// determine whether serverNameOrIP is a name or an IP, and fill _data accordingly
	host.s_addr = inet_addr(serverNameOrIP.c_str());
	if(host.s_addr == INADDR_NONE)
	{
		// it's not an IP address. Hopefully it's a name
		isName = true;
		_data = gethostbyname(serverNameOrIP.c_str());
	}
	else
	{
		// it's an IP
		isName = false;
		_data = gethostbyaddr((const char *)&host, sizeof(struct in_addr), AF_INET);
	}

	if(_data == NULL)
	{
		if(isName)
			throw EasyNetError(h_errno, "EasyAddress:EasyAddress()", "gethostbyname()");
		else
			throw EasyNetError(h_errno, "EasyAddress:EasyAddress()", "gethostbyaddr()");
	}
}

EasyAddress::~EasyAddress(){
	// unsure whether I should free() _data....is that handled by me, or Linux?
	// --- NOTE! If you ever decide to free _data, make sure to add a copy
	// constructor and an operator=!
}

/** Returns the host's human-friendly name */
string EasyAddress::Name() const throw() {
	return string(_data->h_name);
}

/** Returns a list of all the names this host is known by */
vector<string> EasyAddress::Aliases() const throw() {
	vector<string> aliases;
	for(int index = 0; ; index++)
	{		
		if(_data->h_aliases[index] != NULL)
			aliases.push_back(_data->h_aliases[index]);
		else
			break;
	}
	return aliases;
}

/** Returns all the IP addresses that this host is known by */
vector<string> EasyAddress::Addresses() const throw() {
	struct in_addr *addr;
	vector<string> addresses;
	for(int index= 0; ; index++)
	{
		addr = (struct in_addr*)_data->h_addr_list[index];

		if(addr != NULL)
			addresses.push_back(inet_ntoa(*addr));
		else
			break;
	}
	return addresses;
}
