/***************************************************************************
                          ezneterror.h  -  description                              
                             -------------------                                         
    begin                : Tue Aug 17 1999                                           
    copyright            : (C) 1999 by Joe Nelson, and Signal 11
    email                : 3nelsons@pressenter.com                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef EasyNETERROR_H
#define EasyNETERROR_H

#include <errno.h>

#include <string>
#include <exception>
using namespace std;

/**Exception thrown by the classes of the EasyNet library
  *@author Joe Nelson and Signal 11
  */

class EasyNetError : public exception  {
	string _what;
	string _where;
	int _errornum;
public:
	EasyNetError(const int errornum_arg, const string& where_arg, const string& applies_to)
	: _what(""), _where(where_arg), _errornum(errornum_arg) {

		// Current call codes documented:
		// connect(), accept(), bind(), getsockname(), getsockopt(), listen(),
		// gethostbyname(), gethostbyaddr(), socket(), read(), and write()

		_what = applies_to + ": ";

		if(applies_to == "connect()")
		{			
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				case ENOTSOCK:
					_what += "Descriptor not associated to a socket.";
					break;
				case EISCONN:
					_what += "Already connected.";
					break;
				case ECONNREFUSED:
					_what += "Connection refused.";
					break;
				case ETIMEDOUT:
					_what += "Connection timed out.";
					break;
				case ENETUNREACH:
					_what += "Network is unreachable.";
					break;
				case EADDRINUSE:
					_what += "Address already in use.";
					break;
				case EINPROGRESS:
					_what += "Connection in progress.";
					break;
				case EALREADY:
					_what += "Socket is non-blocking and a previous connection attempt has not completed yet";
					break;
				case EAFNOSUPPORT:
					_what += "Address didn't have sa_family set properly.  try AF_INET.";
					break;
				case EACCES:
					_what += "Attempted to connect to a broadcast address without setting the broadcast flag on socket.";
					break;
				default:
					_what += "Unspecified error!";
					break;
			}
	}
	else if(applies_to == "accept()")
	{
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case ENOTSOCK:
					_what += "Was expecting socket descriptor, got file descriptor.";
					break;
				case EOPNOTSUPP:
					_what += "Socket not of type SOCK_STREAM!";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				case EAGAIN:
					_what += "No connections pending.";
					break;
				case EPERM:
					_what += "Local firewall rules prohibit this connection.";
					break;
				case ENOBUFS:
					_what += "Out of memory!";
					break;
				case ENOMEM:
					_what += "Out of memory!";
					break;
				default:
					_what += "Unspecified error!";
					break;
			}
		}
		else if(applies_to == "bind()")
		{
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case ENOTSOCK:
					_what += "Was expecting socket descriptor, got file descriptor.";
					break;
				case EINVAL:
					_what += "Socket already bound or socket not in AF_UNIX family.";
					break;
				case EACCES:
					_what += "Only root can do that.(unsufficient permissions)";
					break;
				// AF_UNIX specific
				case EROFS:
					_what += "Read-only filesystem: bind to socket failed.";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				case ENAMETOOLONG:
					_what += "Address passed is too long.";
					break;
				case ENOENT:
					_what += "File not found.";
					break;
				case ENOMEM:
					_what += "No kernel memory available.";
					break;
				case ENOTDIR:
					_what += "Invalid path: non-directory components found.";
					break;
				case ELOOP:
					_what += "Too many shambolic links.  Consult an excorcist.";
					break;
				default:
					_what += "Unspecified error!";
					break;
			}
		}
		else if(applies_to == "getsockname()")
		{
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case ENOTSOCK:
					_what += "Was expecting socket descriptor, got file descriptor.";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				case ENOBUFS:
					_what += "Out of memory.";
					break;
				default:
					_what += "Unspecified error!";
					break;
			}
		}
		else if(applies_to == "getsockopt()")
		{
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case ENOTSOCK:
					_what += "Was expecting socket descriptor, got file descriptor.";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				case ENOPROTOOPT:
					_what += "Unknown option at level specified. (can you be alittle more vague please?)";
				default:
					_what += "Unspecified error!";
					break;
			}
		}
		else if(applies_to == "listen()")
		{
			switch(_errornum){
				case EBADF:
					_what += "Bad descriptor.";
					break;
				case ENOTSOCK:
					_what += "Was expecting socket descriptor, got file descriptor.";
					break;
				case EOPNOTSUPP:
					_what += "Socket not of the correct type.";
					break;
				default:
					_what += "Unspecified error!";
					break;
			}
		}
 		else if (applies_to == "gethostbyname()" || applies_to == "gethostbyaddr()")
		{
 			switch(_errornum)
			{
 				case HOST_NOT_FOUND:
 					_what += "Host not found.";
	 				break;
	 			case NO_ADDRESS:
	 				_what += "The requested host name is valid but does not have an IP address.";
 					break;
	 			case NO_RECOVERY:
	 				_what += "An unrecoverable error occured while querying nameserver.";
	 				break;
	 			case TRY_AGAIN:
	 				_what += "Temporary error resolving host.  You can try again later.";
	 				break;
	 			default:
	 				_what += "Unspecified error!";
			}
 		}
		else if(applies_to == "socket()")
		{
			switch(_errornum)
			{
				case EPROTONOSUPPORT:
					_what += "Protocol not supported in this domain.";
					break;
				case ENFILE:
					_what += "Not enough kernel memory to allocate a new socket structure.";
					break;
				case EMFILE:
					_what += "Process file table overflow.";
					break;
				case EACCES:
					_what += "Permission to create socket denied.";
					break;
				case ENOMEM:
					_what += "Insufficient memory. Free some resources!";
					break;
				case EINVAL:
					_what += "Unknown protocol, or protocol family not supported.";
					break;
			}
		}
    else if (applies_to == "read()" || applies_to == "write()")
		{
			switch (_errornum)
			{
				case EINTR:
					_what += "Interrupt received, call aborted.";
					break;
				case EAGAIN:
					_what += "This port is set to O_NONBLOCK, and no data is available to read right now.";
					break;
				case EISDIR:
					_what += "You moron, this descriptor points to a directory!";
					break;
				case EIO:
					_what += "unspecified I/O error. This probably isn't good.";
					break;
				case EBADF:
					_what += "invalid or uninitialized socket descriptor.";
					break;
				case EINVAL:
					_what += "Cannot read from this descriptor.";
					break;
				case EFAULT:
					_what += "Address passed is outside of program's address space.";
					break;
				// errors specific to write()
				case EPIPE:
					_what += "Broken pipe.";	// If you get this error, you didn't do your job.
					break;		// go back and install a signal handler and do it right!
				case ENOSPC:
					_what += "Can't write to device - no space left.";
					break;
				default:
					_what += "Unknown error!";
					break;
			}
		}
		else
			_what += "No information available.";
	}
	
	virtual const char* what() const { return _what.c_str(); }
	virtual const char* where() const { return _where.c_str(); }
	virtual const int errornum() const { return _errornum; }
};

#endif
